﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RemoteSQLite
{
    public class DataTableConverter : JsonConverter
    {
        //是否开启自定义反序列化，值为true时，反序列化时会走ReadJson方法，值为false时，不走ReadJson方法，而是默认的反序列化
        public override bool CanRead => true;
        //是否开启自定义序列化，值为true时，序列化时会走WriteJson方法，值为false时，不走WriteJson方法，而是默认的序列化
        public override bool CanWrite => true;

        public override bool CanConvert(Type objectType)
        {
            return typeof(DataTable) == objectType;
        }

        public override object ReadJson(JsonReader reader, Type objectType, object existingValue, JsonSerializer serializer)
        {
            if (reader.TokenType == JsonToken.Null)
            {
                return null;
            }

            if (!(existingValue is DataTable dt))
            {
                // handle typed datasets
                dt = (objectType == typeof(DataTable))
                    ? new DataTable()
                    : (DataTable)Activator.CreateInstance(objectType);
            }

            // DataTable is inside a DataSet
            // populate the name from the property name
            if (reader.TokenType == JsonToken.PropertyName)
            {
                dt.TableName = (string)reader.Value;

                reader.Read();

                if (reader.TokenType == JsonToken.Null)
                {
                    return dt;
                }
            }

            if (reader.TokenType != JsonToken.StartObject)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable. Expected StartObject, {reader.Value}");
            }

            reader.Read();
            CreateColumns(reader, dt, serializer);

            reader.Read();
            CreateRows(reader, dt, serializer);

            reader.Read();
            if (reader.TokenType != JsonToken.EndObject)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable. Expected EndObject, {reader.Value}");
            }

            return dt;
        }
        private static void CreateColumns(JsonReader reader, DataTable dt, JsonSerializer serializer)
        {
            if (reader.TokenType != JsonToken.PropertyName)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Columns Expected PropertyName, {reader.Value}");
            }
            if ((string)reader.Value != "Columns")
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Columns Expected PropertyName = Columns, but {reader.Value}");
            }

            reader.Read();
            if (reader.TokenType != JsonToken.StartObject)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Columns Expected StartObject, {reader.Value}");
            }

            reader.Read();
            while (reader.TokenType != JsonToken.EndObject)
            {
                CreateColumn(reader, dt, serializer);

                reader.Read();
            }
        }
        private static void CreateColumn(JsonReader reader, DataTable dt, JsonSerializer serializer)
        {
            DataColumn dataColumn = new DataColumn();
            if (reader.TokenType != JsonToken.PropertyName)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Columns[] Expected PropertyName for ColumnName, {reader.Value}");
            }
            dataColumn.ColumnName = (string)reader.Value;
            reader.Read();
            if (reader.TokenType != JsonToken.String)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Columns[] Expected String for DataType, {reader.Value}");
            }

            Type type = Type.GetType(reader.Value as string);
            dataColumn.DataType = type;
            dt.Columns.Add(dataColumn);
        }
        private static void CreateRows(JsonReader reader, DataTable dt, JsonSerializer serializer)
        {
            if (reader.TokenType != JsonToken.PropertyName)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Rows Expected PropertyName, {reader.Value}");
            }
            if ((string)reader.Value != "Rows")
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Columns Expected PropertyName = Rows, {reader.Value}");
            }

            reader.Read();
            if (reader.TokenType != JsonToken.StartArray)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Columns Expected StartArray, {reader.Value}");
            }

            reader.Read();
            while (reader.TokenType != JsonToken.EndArray)
            {
                CreateRow(reader, dt, serializer);

                reader.Read();
            }
        }
        private static void CreateRow(JsonReader reader, DataTable dt, JsonSerializer serializer)
        {
            if (reader.TokenType != JsonToken.StartArray)
            {
                throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Rows[] Expected StartArray, {reader.Value}");
            }

            reader.Read();

            DataRow dr = dt.NewRow();
            int column_idx = 0;

            while (reader.TokenType != JsonToken.EndArray)
            {
                if (column_idx >= dt.Columns.Count)
                {
                    throw new JsonSerializationException($"Unexpected JSON token when reading DataTable.Rows[] too many column column_idx={column_idx} ");
                }
                var datatype = dt.Columns[column_idx].DataType;

                object columnValue = (reader.Value != null)
                        ? serializer.Deserialize(reader, datatype) ?? DBNull.Value
                        : DBNull.Value;

                dr[column_idx] = columnValue;
                column_idx++;
                reader.Read();
            }

            dr.EndEdit();
            dt.Rows.Add(dr);
        }
        public override void WriteJson(JsonWriter writer, object value, JsonSerializer serializer)
        {

            //new一个JObject对象,JObject可以像操作对象来操作json
            var jobj_table = new JObject();
            //value参数实际上是你要序列化的Model对象，所以此处直接强转
            var dt = value as DataTable;

            //列信息
            JObject jobject_columns = new JObject();
            foreach (DataColumn col in dt.Columns)
            {
                jobject_columns.Add(col.ColumnName, col.DataType.FullName);
            }
            jobj_table.Add("Columns", jobject_columns);

            //行信息
            JArray jarray_rows = new JArray();
            foreach (DataRow row in dt.Rows)
            {
                JArray jarray_row = new JArray();
                foreach (var v in row.ItemArray)
                {
                    jarray_row.Add(v);
                }
                jarray_rows.Add(jarray_row);
            }
            jobj_table.Add("Rows", jarray_rows);

            //通过ToString()方法把JObject对象转换成json
            //var jsonstr = jobj_table.ToString(serializer.Formatting);
            //调用该方法，把json放进去，最终序列化Model对象的json就是jsonstr，由此，我们就能自定义的序列化对象了

            JsonReader reader = jobj_table.CreateReader();
            writer.WriteToken(reader);
        }
    }

}
